package com.seeyon.apps.collaboration.po;


import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;

import com.seeyon.apps.collaboration.constants.ColConstant;
import com.seeyon.ctp.common.log.CtpLogFactory;
import com.seeyon.ctp.common.po.BasePO;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.json.JSONUtil;

/**
 * This is an object that contains data related to the COL_SUMMARY table.
 *
 *  协同主表 
 *
 * @hibernate.class
 *  table="COL_SUMMARY"
 */
public class ColSummary extends BasePO {

    private static Log LOG = CtpLogFactory.getLog(ColSummary.class);

    /*[IDTCODE MARKER BEGIN]*/


    // fields
    /**
     *  标题
     */
    private String _subject;
    /**
     *  流程ID
     */
    private String _processId;
    /**
     *  协同类型
     */
    private Integer _colType;
    /**
     *  重要程度
     */
    private Integer _importantLevel;
    /**
     *  流程状态
     */
    private Integer _state;
    /**
     *  审核通过/审核不通过
     */
    private Boolean _audited;
    /**
     *  模板ID
     */
    private Long _templeteId;
    /**
     *  结束时间
     */
    private Date _finishDate;
    /**
     *  按工作时间计算超期时间
     */
    private Long _overWorktime;
    /**
     *  转发人
     */
    private String _forwardMember;
    /**
     *  父协同ID
     */
    private Long _parentformSummaryid;
    /**
     *  协同来源（用于显示外部系统名称，如NC）
     */
    private String _source;
    /**
     *  能否修改正文
     */
    private Boolean _canEdit = false;
    /**
     *  是否能归档
     */
    private Boolean _canArchive = false;
    /**
     *  关联项目ID
     */
    private Long _projectId;
    /**
     *  流程规则
     */
    private String _workflowRule;
    /**
     *  主表记录id
     */
    private Long _formRecordid;
    /**
     *  表单视图id
     *  @deprecated
     */
    private Long _formid;

    /**
     *  发起人
     */
    private Long _startMemberId;
    /**
     *  是否自动终止流程
     */
    private Boolean _canAutostopflow = false;
    /**
     *  是否能跟踪
     */
    private Boolean _canTrack = false;
    /**
     *  标志串
     */
    private String _identifier;
    /**
     *  能否编辑附件
     */
    private Boolean _canEditAttachment = false;
    /**
     *  单位ID
     */
    private Long _orgAccountId;
    /**
     *  流程实例ID
     */
    private Long _caseId;
    /**
     *  按自然时间计算运行时间
     */
    private Long _runTime;
    /**
     *  提醒周期
     */
    private Long _remindInterval;
    /**
     *  开始时间
     */
    private Date _startDate;
    /**
     *  按自然时间计算超期时间
     */
    private Long _overTime;
    /**
     *  重复次数
     */
    private Integer _resentTime;
    /**
     *  按工作时间计算运行时间
     */
    private Long _runWorktime;
    /**
     *  流程期限时间段
     */
    private Long _deadline;
    /**
     * 模板设置的督办期限
     */
    private String deadlineTemplate;
    /**
     *  流程期限时间点
     */
    private Date _deadlineDatetime;
    /**
     *  新流程类型
     */
    private Integer _newflowType;
    /**
     *  是否能修改
     */
    private Boolean _canModify = false;
    /**
     *  正文类型
     */
    private String _bodyType;
    /**
     *  归档ID
     */
    private Long _archiveId;
    /**
     *  是否周期提醒
     */
    private Boolean _canDueReminder = false;
    /**
     *  创建时间
     */
    private Date _createDate;
    /**
     *  表单ID
     */
    private Long _formAppid;
    /**
     *  是否核定0：默认值;1核定通过;2核定不能过
     */
    private Integer _vouch;
    /**
     *  部门ID
     */
    private Long _orgDepartmentId;
    /**
     *  是否能转发
     */
    private Boolean _canForward = false;
    /**
     * 是否自动更新标题
     */
    private Boolean updateSubject = false;
    /**
     *  webserivce服务码
     */
    private String _webserviceCode;
    /**
     *  提前提醒
     */
    private Long _advanceRemind;
    /**
     *  督办人员Id串
     */
    private String _supervisorsId;
    /**
     *  督办人员名称字符串
     */
    private String _supervisors;
    /**
     *  督办时间
     */
    private String _awakeDate;
    /**
     *  督办主题
     */
    private String _superviseTitle;

    /**
     *  是否超期
     */
    private Boolean _coverTime = false;

    /**
     * 当前处理人信息
     */
    private String _currentNodesInfo;

    //是否合并处理
    @Deprecated
    private Boolean canMergeDeal;

    //是否自动发起
    private Boolean autoRun;

    private Integer subState;

    /**
     * 流程超期处理类型(0-自动终止，1-自动撤销)
     */
    private Integer processTermType;

    /**
     * 合并处理类型
     */
    private String mergeDealType;
    private Long relationId;

    private String messageRuleId;

    /**
     *  是否能减签
     */
    private Boolean canDeleteNode = true;

    /**
     *  流程密级
     */
    private Integer secretLevel;

    public Integer getPullState() {
        return pullState;
    }

    public void setPullState(Integer pullState) {
        this.pullState = pullState;
    }

    /**
     * 拉取状态
     * @return
     */
    private Integer pullState;

    public Integer getErpState() {
        return erpState;
    }

    public void setErpState(Integer erpState) {
        this.erpState = erpState;
    }

    /**
     * ERP的状态
     * @return
     */
    private Integer erpState;

    public Integer getGeneralState() {
        return generalState;
    }

    public void setGeneralState(Integer generalState) {
        this.generalState = generalState;
    }

    /**
     * 总状态
     * @return
     */
    private Integer generalState;


    public Integer getSecretLevel() {
        return secretLevel;
    }

    public void setSecretLevel(Integer secretLevel) {
        this.secretLevel = secretLevel;
    }
    public String getMessageRuleId() {
        return messageRuleId;
    }

    public void setMessageRuleId(String messageRuleId) {
        this.messageRuleId = messageRuleId;
    }

    public Integer getSubState() {
        return subState;
    }

    public void setSubState(Integer subState) {
        this.subState = subState;
    }

    public Boolean getAutoRun() {
        return autoRun;
    }

    public void setAutoRun(Boolean autoRun) {
        this.autoRun = autoRun;
    }


    //与前一任处理节点相同时，合并处理
    @Deprecated
    private Boolean canAnyMerge;
    /**
     *  附件归档ID
     */
    private Long _attachmentArchiveId;

    /**
     * 预归档
     */
    private String advancePigeonhole;

    private String processNodesInfo;


    private Long permissionAccountId ;
    /**
     * 回复条数
     */
    private Integer replyCounts;


    public Integer getReplyCounts() {
        return replyCounts;
    }

    public void setReplyCounts(Integer replyCounts) {
        //	LOG.info("设置协同回复数属性：id="+this.getId()+",count="+replyCounts);
        this.replyCounts = replyCounts;
    }

    public Long getPermissionAccountId() {
        return permissionAccountId;
    }

    public void setPermissionAccountId(Long permissionAccountId) {
        this.permissionAccountId = permissionAccountId;
    }

    public String getProcessNodesInfo() {
        return processNodesInfo;
    }

    public void setProcessNodesInfo(String processNodesInfo) {
        this.processNodesInfo = processNodesInfo;
    }

    public String getAdvancePigeonhole() {
        return advancePigeonhole;
    }

    public void setAdvancePigeonhole(String advancePigeonhole) {
        this.advancePigeonhole = advancePigeonhole;
    }
    @Deprecated
    public Boolean getCanMergeDeal() {
        if (canMergeDeal == null) {
            return Boolean.FALSE;
        }
        return canMergeDeal;
    }
    @Deprecated
    public void setCanMergeDeal(Boolean isMergeDeal) {
        this.canMergeDeal = isMergeDeal;
    }

    /**
     *  是否超期
     */
    public Boolean isCoverTime () {
        return _coverTime;
    }
    public Boolean getCoverTime () {
        return _coverTime;
    }
    /**
     *  是否超期
     * @param _coverTime the IS_COVER_TIME value
     */
    public void setCoverTime (Boolean _coverTime) {
        this._coverTime = _coverTime;
    }

    /**
     *  标题
     */
    public String getSubject () {
        return _subject;
    }

    /**
     *  标题
     * @param _subject the SUBJECT value
     */
    public void setSubject (String _subject) {
        this._subject = _subject;
    }


    /**
     *  流程ID
     */
    public String getProcessId () {
        return _processId;
    }

    /**
     *  流程ID
     * @param _processId the PROCESS_ID value
     */
    public void setProcessId (String _processId) {
        this._processId = _processId;
    }


    /**
     *  协同类型
     */
    public Integer getColType () {
        return _colType;
    }

    /**
     *  协同类型
     * @param _colType the COL_TYPE value
     */
    public void setColType (Integer _colType) {
        this._colType = _colType;
    }


    /**
     *  重要程度
     */
    public Integer getImportantLevel () {
        return _importantLevel;
    }

    /**
     *  重要程度
     * @param _importantLevel the IMPORTANT_LEVEL value
     */
    public void setImportantLevel (Integer _importantLevel) {
        this._importantLevel = _importantLevel;
    }


    /**
     *  流程状态
     */
    public Integer getState () {
        return _state;
    }

    /**
     *  流程状态
     * @param _state the STATE value
     */
    public void setState (Integer _state) {
        this._state = _state;
    }


    /**
     *  审核通过/审核不通过
     */
    public Boolean isAudited () {
        return _audited;
    }
    public Boolean getAudited () {
        return _audited;
    }
    /**
     *  审核通过/审核不通过
     * @param _audited the IS_AUDITED value
     */
    public void setAudited (Boolean _audited) {
        this._audited = _audited;
    }


    /**
     *  模板ID
     */
    public Long getTempleteId () {
        return _templeteId;
    }

    /**
     *  模板ID
     * @param _templeteId the TEMPLETE_ID value
     */
    public void setTempleteId (Long _templeteId) {
        this._templeteId = _templeteId;
    }


    /**
     *  结束时间
     */
    public Date getFinishDate () {
        return _finishDate;
    }

    /**
     *  结束时间
     * @param _finishDate the FINISH_DATE value
     */
    public void setFinishDate (Date _finishDate) {
        this._finishDate = _finishDate;
    }


    /**
     *  按工作时间计算超期时间
     */
    public Long getOverWorktime () {
        return _overWorktime;
    }

    /**
     *  按工作时间计算超期时间
     * @param _overWorktime the OVER_WORKTIME value
     */
    public void setOverWorktime (Long _overWorktime) {
        this._overWorktime = _overWorktime;
    }


    /**
     *  转发人
     */
    public String getForwardMember () {
        return _forwardMember;
    }

    /**
     *  转发人
     * @param _forwardMember the FORWARD_MEMBER value
     */
    public void setForwardMember (String _forwardMember) {
        this._forwardMember = _forwardMember;
    }


    /**
     *  父协同ID
     */
    public Long getParentformSummaryid () {
        return _parentformSummaryid;
    }

    /**
     *  父协同ID
     * @param _parentformSummaryid the PARENTFORM_SUMMARYID value
     */
    public void setParentformSummaryid (Long _parentformSummaryid) {
        this._parentformSummaryid = _parentformSummaryid;
    }


    /**
     *  协同来源（用于显示外部系统名称，如NC）
     */
    public String getSource () {
        return _source;
    }

    /**
     *  协同来源（用于显示外部系统名称，如NC）
     * @param _source the SOURCE value
     */
    public void setSource (String _source) {
        this._source = _source;
    }


    /**
     *  能否修改正文
     */
    public Boolean getCanEdit () {
        return _canEdit;
    }

    /**
     *  能否修改正文
     * @param _canEdit the CAN_EDIT value
     */
    public void setCanEdit (Boolean _canEdit) {
        this._canEdit = _canEdit;
    }


    /**
     *  是否能归档
     */
    public Boolean getCanArchive () {
        return _canArchive;
    }
    /**
     *  是否能归档
     * @param _canArchive the CAN_ARCHIVE value
     */
    public void setCanArchive (Boolean _canArchive) {
        this._canArchive = _canArchive;
    }


    /**
     *  关联项目ID
     */
    public Long getProjectId () {
        return _projectId;
    }

    /**
     *  关联项目ID
     * @param _projectId the PROJECT_ID value
     */
    public void setProjectId (Long _projectId) {
        this._projectId = _projectId;
    }


    /**
     *  流程规则
     */
    @Deprecated
    public String getWorkflowRule () {
        return _workflowRule;
    }

    /**
     *  流程规则
     * @param _workflowRule the WORKFLOW_RULE value
     */
    @Deprecated
    public void setWorkflowRule (String _workflowRule) {
        this._workflowRule = _workflowRule;
    }


    /**
     *  主表记录id
     */
    public Long getFormRecordid () {
        return _formRecordid;
    }

    /**
     *  主表记录id
     * @param _formRecordid the FORM_RECORDID value
     */
    public void setFormRecordid (Long _formRecordid) {
        this._formRecordid = _formRecordid;
    }


    /**
     *  表单视图id
     *
     */
    @Deprecated
    public Long getFormid () {
        return _formid;
    }

    /**
     *  表单视图id
     * @param _formid the FORMID value
     */
    @Deprecated
    public void setFormid (Long _formid) {
        this._formid = _formid;
    }


    /**
     *  发起人
     */
    public Long getStartMemberId () {
        return _startMemberId;
    }

    /**
     *  发起人
     * @param _startMemberId the START_MEMBER_ID value
     */
    public void setStartMemberId (Long _startMemberId) {
        this._startMemberId = _startMemberId;
    }


    /**
     *  是否自动终止流程
     */
    public Boolean getCanAutostopflow () {
        return _canAutostopflow;
    }

    /**
     *  是否自动终止流程
     * @param _canAutostopflow the CAN_AUTOSTOPFLOW value
     */
    public void setCanAutostopflow (Boolean _canAutostopflow) {
        this._canAutostopflow = _canAutostopflow;
    }


    /**
     *  是否能跟踪
     */
    public Boolean getCanTrack () {
        return _canTrack;
    }

    /**
     *  是否能跟踪
     * @param _canTrack the CAN_TRACK value
     */
    public void setCanTrack (Boolean _canTrack) {
        this._canTrack = _canTrack;
    }


    /**
     *  标志串
     */
    public String getIdentifier () {
        return _identifier;
    }

    /**
     *  标志串
     * @param _identifier the IDENTIFIER value
     */
    public void setIdentifier (String _identifier) {
        this._identifier = _identifier;
    }


    /**
     *  能否编辑附件
     */
    public Boolean getCanEditAttachment () {
        return _canEditAttachment;
    }

    /**
     *  能否编辑附件
     * @param _canEditAttachment the CAN_EDIT_ATTACHMENT value
     */
    public void setCanEditAttachment (Boolean _canEditAttachment) {
        this._canEditAttachment = _canEditAttachment;
    }


    /**
     *  单位ID
     */
    public Long getOrgAccountId () {
        return _orgAccountId;
    }

    /**
     *  单位ID
     * @param _orgAccountId the ORG_ACCOUNT_ID value
     */
    public void setOrgAccountId (Long _orgAccountId) {
        this._orgAccountId = _orgAccountId;
    }


    /**
     *  流程实例ID
     */
    public Long getCaseId () {
        return _caseId;
    }

    /**
     *  流程实例ID
     * @param _caseId the CASE_ID value
     */
    public void setCaseId (Long _caseId) {
        this._caseId = _caseId;
    }


    /**
     *  按自然时间计算运行时间
     */
    public Long getRunTime () {
        return _runTime;
    }

    /**
     *  按自然时间计算运行时间
     * @param _runTime the RUN_TIME value
     */
    public void setRunTime (Long _runTime) {
        this._runTime = _runTime;
    }


    /**
     *  提醒周期
     */
    public Long getRemindInterval () {
        return _remindInterval;
    }

    /**
     *  提醒周期
     * @param _remindInterval the REMIND_INTERVAL value
     */
    public void setRemindInterval (Long _remindInterval) {
        this._remindInterval = _remindInterval;
    }


    /**
     *  开始时间
     */
    public Date getStartDate () {
        return _startDate;
    }

    /**
     *  开始时间
     * @param _startDate the START_DATE value
     */
    public void setStartDate (Date _startDate) {
        this._startDate = _startDate;
    }


    /**
     *  按自然时间计算超期时间
     */
    public Long getOverTime () {
        return _overTime;
    }

    /**
     *  按自然时间计算超期时间
     * @param _overTime the OVER_TIME value
     */
    public void setOverTime (Long _overTime) {
        this._overTime = _overTime;
    }


    /**
     *  重复次数
     */
    public Integer getResentTime () {
        return _resentTime;
    }

    /**
     *  重复次数
     * @param _resentTime the RESENT_TIME value
     */
    public void setResentTime (Integer _resentTime) {
        this._resentTime = _resentTime;
    }


    /**
     *  按工作时间计算运行时间
     */
    public Long getRunWorktime () {
        return _runWorktime;
    }

    /**
     *  按工作时间计算运行时间
     * @param _runWorktime the RUN_WORKTIME value
     */
    public void setRunWorktime (Long _runWorktime) {
        this._runWorktime = _runWorktime;
    }


    /**
     *  处理期限
     */
    public Long getDeadline () {
        return _deadline;
    }

    /**
     *  处理期限
     * @param _deadline the DEADLINE value
     */
    public void setDeadline (Long _deadline) {
        this._deadline = _deadline;
    }


    /**
     *  新流程类型
     */
    public Integer getNewflowType () {
        if(_newflowType == null){
            return ColConstant.NewflowType.main.ordinal();
        }
        else{
            return _newflowType;
        }
    }

    /**
     *  新流程类型
     * @param _newflowType the NEWFLOW_TYPE value
     */
    public void setNewflowType (Integer _newflowType) {
        this._newflowType = _newflowType;
    }


    /**
     *  是否能修改 流程
     */
    public Boolean getCanModify () {
        return _canModify;
    }

    /**
     *  是否能修改 流程
     * @param _canModify the CAN_MODIFY value
     */
    public void setCanModify (Boolean _canModify) {
        this._canModify = _canModify;
    }


    /**
     *  正文类型
     */
    public String getBodyType () {
        return _bodyType;
    }

    /**
     *  正文类型
     * @param _bodyType the BODY_TYPE value
     */
    public void setBodyType (String _bodyType) {
        this._bodyType = _bodyType;
    }


    /**
     *  归档ID
     */
    public Long getArchiveId () {
        return _archiveId;
    }

    /**
     *  归档ID
     * @param _archiveId the ARCHIVE_ID value
     */
    public void setArchiveId (Long _archiveId) {
        this._archiveId = _archiveId;
    }

    /**
     *  附件归档ID
     */
    public Long getAttachmentArchiveId () {
        return _attachmentArchiveId;
    }

    /**
     *  附件归档ID
     * @param _archiveId the ARCHIVE_ID value
     */
    public void setAttachmentArchiveId (Long _attachmentArchiveId) {
        this._attachmentArchiveId = _attachmentArchiveId;
    }

    /**
     *  是否周期提醒
     */
    public Boolean getCanDueReminder () {
        return _canDueReminder;
    }

    /**
     *  是否周期提醒
     * @param _canDueReminder the CAN_DUE_REMINDER value
     */
    public void setCanDueReminder (Boolean _canDueReminder) {
        this._canDueReminder = _canDueReminder;
    }


    /**
     *  创建时间
     */
    public Date getCreateDate () {
        return _createDate;
    }

    /**
     *  创建时间
     * @param _createDate the CREATE_DATE value
     */
    public void setCreateDate (Date _createDate) {
        this._createDate = _createDate;
    }


    /**
     *  表单ID
     */
    public Long getFormAppid () {
        return _formAppid;
    }

    /**
     *  表单ID
     * @param _formAppid the FORM_APPID value
     */
    public void setFormAppid (Long _formAppid) {
        this._formAppid = _formAppid;
    }


    /**
     *  是否核定0：默认值;1核定通过;2核定不能过
     */
    public Integer getVouch () {
        return _vouch;
    }

    /**
     *  是否核定0：默认值;1核定通过;2核定不能过
     * @param _vouch the IS_VOUCH value
     */
    public void setVouch (Integer _vouch) {
        this._vouch = _vouch;
    }


    /**
     *  部门ID
     */
    public Long getOrgDepartmentId () {
        return _orgDepartmentId;
    }

    /**
     *  部门ID
     * @param _orgDepartmentId the ORG_DEPARTMENT_ID value
     */
    public void setOrgDepartmentId (Long _orgDepartmentId) {
        this._orgDepartmentId = _orgDepartmentId;
    }


    /**
     *  是否能转发
     */
    public Boolean getCanForward () {
        return _canForward;
    }

    /**
     *  是否能转发
     * @param _canForward the CAN_FORWARD value
     */
    public void setCanForward (Boolean _canForward) {
        this._canForward = _canForward;
    }


    /**
     *  webserivce服务码
     */
    public String getWebserviceCode () {
        return _webserviceCode;
    }

    /**
     *  webserivce服务码
     * @param _webserviceCode the WEBSERVICE_CODE value
     */
    public void setWebserviceCode (String _webserviceCode) {
        this._webserviceCode = _webserviceCode;
    }


    /**
     *  提前提醒
     */
    public Long getAdvanceRemind () {
        return _advanceRemind;
    }

    /**
     *  提前提醒
     * @param _advanceRemind the ADVANCE_REMIND value
     */
    public void setAdvanceRemind (Long _advanceRemind) {
        this._advanceRemind = _advanceRemind;
    }

    public String getSupervisorsId() {
        return _supervisorsId;
    }

    public void setSupervisorsId(String _supervisorsId) {
        this._supervisorsId = _supervisorsId;
    }

    public String getSupervisors() {
        return _supervisors;
    }

    public void setSupervisors(String _supervisors) {
        this._supervisors = _supervisors;
    }

    public String getAwakeDate() {
        return _awakeDate;
    }

    public void setAwakeDate(String _awakeDate) {
        this._awakeDate = _awakeDate;
    }

    public String getSuperviseTitle() {
        return _superviseTitle;
    }

    public void setSuperviseTitle(String _superviseTitle) {
        this._superviseTitle = _superviseTitle;
    }

    public Date getDeadlineDatetime() {
        return _deadlineDatetime;
    }

    public void setDeadlineDatetime(Date _deadlineDatetime) {
        this._deadlineDatetime = _deadlineDatetime;
    }

    public String getCurrentNodesInfo() {
        return _currentNodesInfo;
    }

    public void setCurrentNodesInfo(String currentNodesInfo) {
        this._currentNodesInfo = currentNodesInfo;
    }
    @Deprecated
    public Boolean getCanAnyMerge() {
        if (canAnyMerge == null) {
            return Boolean.FALSE;
        }
        return canAnyMerge;
    }
    @Deprecated
    public void setCanAnyMerge(Boolean canAnyMerge) {
        this.canAnyMerge = canAnyMerge;
    }

    public Boolean getUpdateSubject() {
        return updateSubject;
    }

    public void setUpdateSubject(Boolean updateSubject) {
        this.updateSubject = updateSubject;
    }

    public Integer getProcessTermType() {
        return processTermType;
    }

    public void setProcessTermType(Integer processTermType) {
        this.processTermType = processTermType;
    }



    public String getDeadlineTemplate() {
        return deadlineTemplate;
    }

    public void setDeadlineTemplate(String deadlineTemplate) {
        this.deadlineTemplate = deadlineTemplate;
    }

    public String getMergeDealType() {
        if(Strings.isBlank(mergeDealType)){
            Map<String,String> map = new HashMap<String,String>();
            if(getCanMergeDeal()){
                map.put("PRE_DEAL_MERGE", "3");
            }
            if(getCanAnyMerge()){
                map.put("START_MERGE", "2");
                map.put("DEAL_MERGE", "4");
            }
            return JSONUtil.toJSONString(map);
        }
        return mergeDealType;
    }

    public void setMergeDealType(String mergeDealType) {
        this.mergeDealType = mergeDealType;
    }

    public Long getRelationId() {
        return relationId;
    }

    public void setRelationId(Long relationId) {
        this.relationId = relationId;
    }

    public Boolean getCanDeleteNode() {
        // 如果canDeleteNode为null，返回默认值true
        // xml反序列化时，框架用的unsafe.allocateInstance生成实例，不会调用构造器，所以上方默认值无效
        return canDeleteNode == null ? true : canDeleteNode;
    }

    public void setCanDeleteNode(Boolean canDeleteNode) {
        this.canDeleteNode = canDeleteNode;
    }
}