package com.seeyon.apps.ocip.exchange.collaboration.controller;

import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.seeyon.apps.agent.bo.AgentModel;
import com.seeyon.apps.agent.bo.MemberAgentBean;
import com.seeyon.apps.ocip.organization.bo.OcipUser;
import com.seeyon.apps.ocip.util.AdvisorUtil;
import com.seeyon.apps.ocip.util.CommonUtil;
import com.seeyon.apps.ocip.util.OrgUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.authenticate.domain.UserHelper;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.controller.BaseController;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.privilege.manager.PrivilegeMenuManager;
import com.seeyon.ctp.util.Base64;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.annotation.NeedlessCheckLogin;
import com.seeyon.ctp.util.json.JSONUtil;
import com.seeyon.ocip.common.org.OrgMember;
import com.seeyon.ocip.common.organization.IOrganizationManager;
import org.apache.log4j.Logger;
import org.springframework.web.servlet.ModelAndView;
import www.seeyon.com.utils.UUIDUtil;

/**
 * com.seeyon.apps.ocip.exchange.collaboration.controller.CollViewController
 * @author Administrator
 *
 */
@NeedlessCheckLogin
public class CollViewController extends BaseController {
    private static final Logger LOGGER = Logger.getLogger(CollViewController.class);
    private OrgManager orgManager;
    private PrivilegeMenuManager privilegeMenuManager;

    /**
     * 对给定的字符串进行base64解码操作
     */
    public static String decodeData(String inputData) {
        try {
            if (null == inputData) {
                return null;
            }
            return new String(Base64.decodeBase64(inputData.getBytes()), "UTF-8");
        } catch (UnsupportedEncodingException e) {
            LOGGER.error(inputData, e);
        }

        return null;
    }

    public void setPrivilegeMenuManager(PrivilegeMenuManager privilegeMenuManager) {
        this.privilegeMenuManager = privilegeMenuManager;
    }

    public void setOrgManager(OrgManager orgManager) {
        this.orgManager = orgManager;
    }

    /**
     * 新建协同页面
     *
     * @param request
     * @param response
     * @return
     * @throws Exception
     * @author libing
     */
    public ModelAndView index(HttpServletRequest request, HttpServletResponse response) throws Exception {
        String decodeAffairId = request.getParameter("affairId");
        String ticket = request.getParameter("ticket");
        String isAgent = request.getParameter("isAgent");
        String openFrom = request.getParameter("openFrom");
        V3xOrgMember member = null;
        ModelAndView modelAndView = null;
        try {
            // 单点登录的ID
            String ticketStr = decodeData(ticket);
            String affairId = decodeData(decodeAffairId);
            AffairManager viewAffairManager = (AffairManager) AppContext.getBean("affairManager");
            if (Strings.isNotBlank(ticketStr)) {
                member = orgManager.getMemberById(Long.parseLong(ticketStr));
                if (OrgUtil.isPlatformEntity(member)) {
                    boolean ocipAdvisorEnable = AdvisorUtil.isOcipAdvisorEnable();
                    try {
                        IOrganizationManager organizationManager = (IOrganizationManager) AppContext.getBean("organizationManager");
                        OrgMember orgMember = organizationManager.getLocalMember(ticketStr);
                        if (orgMember != null) {
                            String objectId = orgMember.getObjectId();
                            // 只查询本地人员
                            AdvisorUtil.setOcipAdvisorEnable(false);
                            V3xOrgMember localMember = orgManager.getMemberById(Long.parseLong(objectId));
                            if (localMember != null) {
                                member = localMember;
                            }
                        }
                    } catch (Exception e) {
                        LOGGER.error("登录本地人员失败", e);
                    } finally {
                        AdvisorUtil.setOcipAdvisorEnable(ocipAdvisorEnable);
                    }
                }
            }

            if (member != null) {
                User currentUser = AppContext.getCurrentUser();
                // 已经使用这个人员登录了。就不需要登录了。
                if (currentUser == null || !currentUser.getId().equals(member.getId())) {
                    User user;
                    if (OrgUtil.isPlatformEntity(member)) {
                        user = new OcipUser();
                    } else {
                        user = new User();
                    }
                    user.setId(member.getId());
                    user.setLoginAccount(member.getOrgAccountId());
                    user.setAccountId(member.getOrgAccountId());
                    user.setDepartmentId(member.getOrgDepartmentId());
                    user.setName(member.getName());
                    user.setLoginName(String.valueOf(member.getId()));//loginName设为平台人员ID，避免冲突
                    user.setAuditAdmin(true);
                    user.setLocale(Locale.SIMPLIFIED_CHINESE);
                    //标记为外系统单点登录来的请求
                    user.setUserSSOFrom("other");
                    CommonUtil.setCurrentUser(user);
                    UserHelper.setResourceJsonStr(JSONUtil.toJSONString(privilegeMenuManager.getResourceCode(member.getId(), member.getOrgAccountId())));
                }
            } else {
                response.setContentType("text/html;charset=UTF-8");
                PrintWriter out = response.getWriter();
                out.write("获取人员信息失败");
                out.close();
                return null;
            }
            CtpAffair affair = viewAffairManager.get(Long.parseLong(affairId));
            //处理代理
            if ("true".equals(isAgent)) {
                Long agentObjIdLong = affair.getMemberId();
                //代理人对象
                Long agentId = UUIDUtil.getUUIDLong();
                AgentModel agentObject = new AgentModel();
                agentObject.setId(agentId);
                agentObject.setAgentId(member.getId());
                agentObject.setAgentToId(agentObjIdLong);
                Timestamp now = new Timestamp(new Date().getTime());
                Calendar canlendar = Calendar.getInstance();
                //临时授予30分钟代理权限
                canlendar.add(Calendar.MINUTE, 30);
                agentObject.setCreateDate(now);
                agentObject.setStartDate(now);
                agentObject.setEndDate(new Timestamp(canlendar.getTimeInMillis()));
                agentObject.setAgentOption("11111010000000000000");

                MemberAgentBean.getInstance().put(member.getId(), agentObject, null);
                MemberAgentBean.getInstance().put(agentObjIdLong, null, agentObject);
            }
            String linkUrl = "";
            if (Strings.isNotBlank(openFrom)) {
                linkUrl = "/collaboration/collaboration.do?" + "method=summary&isFromHome=true&openFrom=" + openFrom + "&app=1&affairId=" + affairId + "&ocipExchange=1&ticketStr=" + ticketStr;
            } else if (affair != null && affair.getState() == StateEnum.col_pending.key()) {
                linkUrl = "/collaboration/collaboration.do?" + "method=summary&isFromHome=true&openFrom=listPending&app=1&affairId=" + affairId;
            } else if (affair != null && affair.getState() == StateEnum.col_done.key()) {
                linkUrl = "/collaboration/collaboration.do?" + "method=summary&openFrom=listDone&affairId=" + affairId;
            } else if (affair != null && affair.getState() == StateEnum.col_cancel.key()) {
                linkUrl = "/collaboration/collaboration.do?" + "method=summary&isFromHome=true&openFrom=stepBackRecord&app=1&affairId=" + affairId;
            } else {
                linkUrl = "/collaboration/collaboration.do?" + "method=summary&isFromHome=true&openFrom=listPending&app=1&affairId=" + affairId;
            }
            modelAndView = new ModelAndView("common/redirect", "redirectURL", linkUrl);
        } catch (Exception e) {

        }
        return modelAndView;
    }
}
