package com.seeyon.apps.ocip.exchange.collaboration.listener;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.seeyon.apps.collaboration.enums.CollaborationEnum;
import com.seeyon.apps.collaboration.listener.WorkFlowEventListener;
import com.seeyon.apps.collaboration.manager.ColManager;
import com.seeyon.apps.collaboration.po.ColSummary;
import com.seeyon.apps.collaboration.util.ColUtil;
import com.seeyon.apps.ocip.OCIPConstants;
import com.seeyon.apps.ocip.exchange.collaboration.manager.CollaborationExchangeManager;
import com.seeyon.apps.ocip.exchange.collaboration.manager.OCIPBizManager;
import com.seeyon.apps.ocip.util.OrgUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.content.affair.AffairData;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.content.affair.constants.SubStateEnum;
import com.seeyon.ctp.common.content.comment.Comment;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.ParamUtil;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.UniqueList;
import com.seeyon.ctp.workflow.event.AbstractEventListener;
import com.seeyon.ctp.workflow.event.EventDataContext;
import com.seeyon.ocip.common.org.OcipOrgMember;
import com.seeyon.ocip.common.organization.IOrganizationManager;
import com.seeyon.ocip.common.utils.Global;
import com.seeyon.ocip.exchange.model.col.ColOperation;
import net.joinwork.bpm.engine.wapi.WorkItem;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import static com.seeyon.apps.collaboration.listener.WorkFlowEventListener.ASSIGNED_AFFAIRS;

public class OCIPWorkFlowEventListener extends AbstractEventListener {
    public static final String CtpAffairConstant = "CtpAffair";
    private static Log log = LogFactory.getLog(OCIPWorkFlowEventListener.class);
    private boolean hasMutiPlugin = Boolean.valueOf(AppContext.getSystemProperty(OCIPConstants.HAS_MUTI_PLUGIN));//是否开启多G6功能
    private AffairManager affairManager;
    private ColManager colManager;
    private OrgManager orgManager;
    private IOrganizationManager organizationManager;
    private CollaborationExchangeManager collaborationExchangeManager;
    private OCIPBizManager ocipBizManager;

    public void setOrgManager(OrgManager orgManager) {
        this.orgManager = orgManager;
    }

    public IOrganizationManager getOrganizationManager() {
        return organizationManager;
    }

    public void setOrganizationManager(IOrganizationManager organizationManager) {
        this.organizationManager = organizationManager;
    }

    public void setCollaborationExchangeManager(CollaborationExchangeManager collaborationExchangeManager) {
        this.collaborationExchangeManager = collaborationExchangeManager;
    }

    public void setAffairManager(AffairManager affairManager) {
        this.affairManager = affairManager;
    }

    public void setColManager(ColManager colManager) {
        this.colManager = colManager;
    }

    public void setOcipBizManager(OCIPBizManager ocipBizManager) {
        this.ocipBizManager = ocipBizManager;
    }

    @Override
    public int getSortOrder() {
        return 999;
    }

    @Override
    public String getModuleName() {
        return "collaboration";
    }

    @Override
    public boolean onWorkflowAssigned(List<EventDataContext> contextList) {
        if (Strings.isNotEmpty(contextList) && AppContext.hasPlugin("ocip")) {
            try {
                String processIdStr = contextList.get(0).getProcessId();
                AffairData affairData = (AffairData) contextList.get(0).getBusinessData(EventDataContext.CTP_AFFAIR_DATA);
                if (affairData == null && Strings.isNotBlank(processIdStr) && Strings.isDigits(processIdStr)) {
                    Long processId = Long.parseLong(processIdStr);
                    try {
                        ColSummary summary = colManager.getColSummaryByProcessId(processId);
                        if (null != summary) {
                            affairData = ColUtil.getAffairData(summary);
                        }
                    } catch (Exception e) {
                        log.error("", e);
                    }
                }
                if (affairData == null) {
                    return true;
                }
                List<CtpAffair> affairList = affairData.getAffairList();
                if (affairList == null || affairList.isEmpty()) {
                    affairList = (List<CtpAffair>) contextList.get(0).getBusinessData(ASSIGNED_AFFAIRS);
                    if (affairList == null || affairList.isEmpty()) {
                        List<Long> items = new UniqueList<Long>();
                        for (EventDataContext context : contextList) {
                            for (WorkItem item : context.getWorkitemLists()) {
                                items.add(item.getId());
                            }
                        }
                        affairList = affairManager.getAffairBySubObjectIds(items);
                    }
                    affairData.setAffairList(affairList);
                }
                AffairData ocipAffairData = (AffairData) BeanUtils.cloneBean(affairData);
                ColSummary summary = null;
                if (Strings.isNotEmpty(affairList)) {
                    // 回退生成
                    Object summaryObj = contextList.get(0).getBusinessData(WorkFlowEventListener.COLSUMMARY_CONSTANT);
                    if (summaryObj != null) {
                        summary = (ColSummary) summaryObj;
                    } else {
                        try {
                            summary = colManager.getColSummaryById(affairData.getModuleId());
                        } catch (Exception e) {
                            log.error("获取协同主表失败", e);
                        }
                    }
                    // 存在事务缓存,本来不是pengding的他的status也=3,这样就可以过滤了
                    List<CtpAffair> pendingList = affairManager.getAffairs(summary.getId(), StateEnum.col_pending);
                    Set<Long> pendingIds = new HashSet<Long>();
                    for (CtpAffair pAff : pendingList) {
                        try {
                            if (pAff.getId() == affairData.getBusinessData().get("currentAffairId"))
                                continue;
                        } catch (Exception e) {
                            log.error(e.getMessage(), e);
                        }
                        pendingIds.add(pAff.getId());
                    }
                    List<CtpAffair> list = ocipBizManager.getAffairList4ExchangeByPendingIds(affairList, pendingIds,
                            ColUtil.isNotBlank(contextList.get(0).getAddedFromId()) ? Long.valueOf(contextList.get(0).getAddedFromId()) : null);
                    ocipAffairData.setAffairList(list);
                }
                if (ocipAffairData.getAffairList() != null && ocipAffairData.getAffairList().size() > 0 && ocipAffairData.getAffairList().get(0).getApp() == ApplicationCategoryEnum.collaboration.getKey()) {
                    collaborationExchangeManager.handCollaborationAffair(ocipAffairData, ColOperation.TODO, summary);
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return true;
    }

    @Override
    public boolean onWorkitemFinished(EventDataContext context) {
        if (AppContext.hasPlugin("ocip")) {
            try {

                AffairData affairData = (AffairData) context.getBusinessData(EventDataContext.CTP_AFFAIR_DATA);
                CtpAffair affair = affairManager.getAffairBySubObjectId(context.getWorkItem().getId());
                affair.setState(StateEnum.col_done.key());
                List<CtpAffair> list = affairData.getAffairList();
                AffairData cloneAData = null;
                try {
                    cloneAData = (AffairData) BeanUtils.cloneBean(affairData);
                } catch (Exception e) {
                    log.error("AffairData拷贝失败", e);
                }
                if (Strings.isEmpty(list)) {
                    list = new UniqueList<CtpAffair>();
                }
                if (affair.getCompleteTime() == null) {//设置处理时间
                    affair.setCompleteTime(new Timestamp(System.currentTimeMillis()));
                }
                if (!AppContext.getCurrentUser().getId().equals(affair.getMemberId())) {
                    affair.setTransactorId(AppContext.getCurrentUser().getId());
                }
                list.add(affair);
                ColSummary summary = null;
                ColSummary cloneSummary = null;
                Object summaryObj = context.getBusinessData(WorkFlowEventListener.COLSUMMARY_CONSTANT);
                if (summaryObj != null) {
                    summary = (ColSummary) summaryObj;
                } else {
                    try {
                        summary = colManager.getColSummaryById(affairData.getModuleId());
                    } catch (Exception e) {
                        log.error("获取协同主表失败", e);
                    }
                }
                if (affair.getApp() == ApplicationCategoryEnum.collaboration.getKey()) {
                    try {
                        cloneAData.setAffairList(ocipBizManager.getAffairList4Exchange(list));
                        cloneSummary = (ColSummary) BeanUtils.cloneBean(summary);
                    } catch (Exception e) {
                        log.error("获取交换数据失败", e);
                    }
                    if (cloneAData.getAffairList() != null && cloneAData.getAffairList().size() > 0) {
                        collaborationExchangeManager.handCollaborationAffair(cloneAData, ColOperation.DONE, cloneSummary);
                    }
                }

            } catch (BusinessException e) {
                log.error(e.getMessage(), e);
            }
        }
        return true;
    }

    // 退回
    @Override
    public boolean onWorkitemCanceled(EventDataContext context) {
        if (AppContext.hasPlugin("ocip")) {
            try {
//                User user = AppContext.getCurrentUser();
                Integer _operationType = (Integer) context.getBusinessData().get("operationType");
                int operationType = _operationType == null ? AUTODELETE : _operationType;
//                V3xOrgMember cMember = orgManager.getMemberById(user.getId());
//                List<String> objectList = new ArrayList<String>();
//                objectList.add(user.getId().toString());
//                String resource = Global.getConfig("sysCode");
//                Map<String, List<OrgMember>> mappingMember = new HashMap<String, List<OrgMember>>();
//                if (organizationManager != null && hasMutiPlugin && OnlineChecker.isOnline()) {
//                    mappingMember = organizationManager.getMemberByObjectIds(objectList, resource);
//                }
//                List<OrgMember> mappings = mappingMember.get(user.getId().toString());
//                if (operationType != TAKE_BACK && !OrgUtil.isPlatformEntity(cMember) && (mappings == null || mappings.isEmpty())) {
//                    return true;
//                }
                // 是否为竞争执行
                this.delTakeBack(operationType, context);
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return true;
    }

    private boolean delTakeBack(int operationType, EventDataContext context) throws Exception {
        boolean isCompetition = (operationType == WorkFlowEventListener.COMMONDISPOSAL || operationType == WorkFlowEventListener.ZCDB || operationType == WorkFlowEventListener.AUTOSKIP || operationType == WorkFlowEventListener.SPECIAL_BACK_SUBMITTO)
                && "competition".equals(context.getProcessMode());
        // 退回||取回||指定回退：流程重走
        boolean isBack = operationType == WorkFlowEventListener.WITHDRAW || operationType == WorkFlowEventListener.TAKE_BACK || operationType == WorkFlowEventListener.SPECIAL_BACK_RERUN;
        boolean isDelete = operationType == AUTODELETE;
        if (isCompetition || isBack || isDelete) {
            List<WorkItem> workitemLists = context.getWorkitemLists();
            List<CtpAffair> affairs = new UniqueList<CtpAffair>();
            for (WorkItem item : workitemLists) {
                CtpAffair affair = affairManager.getAffairBySubObjectId(item.getId());
                if (affair != null) {
                    // 直接查出来的对象，不能直接改变，不然会引起update操作，如果前面使用的hql进行update，这里就会导致hql更新无效
                    affairs.add(affair);
                }
            }
            //没有跨系统数据，直接返回
            affairs = ocipBizManager.getAffairList4Exchange(affairs);
            if (affairs.isEmpty()) {
                return true;
            }
            for (CtpAffair affair : affairs) {
                if (isBack && affair.getBackFromId() == null) {//设置回退人
                    User currentUser = AppContext.getCurrentUser();
                    OcipOrgMember platformMember = OrgUtil.getPlatformMember(currentUser.getId());
                    if (platformMember != null) {//管理员不在ocip平台
                        affair.setBackFromId(Long.parseLong(platformMember.getId()));
                    } else {//管理员不在ocip平台直接设置管理员ID（管理员id在数据库初始化时已预设，可跨系统通用）
                        affair.setBackFromId(currentUser.getId());
                    }
                }
                //竞争执行需要把执行人传过去，暂时用senderId传
                if (isCompetition) {
                    affair.setSenderId(Long.valueOf(context.getCurrentUserId()));
                }
            }
            AffairData affairData = (AffairData) context.getBusinessData(EventDataContext.CTP_AFFAIR_DATA);
            if (affairData == null) {
                affairData = new AffairData();
                affairData.setIsSendMessage(true);
            }
            AffairData cloneAffairData = (AffairData) BeanUtils.cloneBean(affairData);

            cloneAffairData.setAffairList(affairs);
            ColSummary summary = null;
            Object summaryObj = context.getBusinessData(WorkFlowEventListener.COLSUMMARY_CONSTANT);
            if (summaryObj != null) {
                summary = (ColSummary) summaryObj;
            } else {
                try {
                    summary = colManager.getColSummaryById(affairs.get(0).getObjectId());
                } catch (Exception e) {
                    log.error("获取协同主表失败", e);
                }
            }
            //复制colSummary对象
            ColSummary cloneSummary = (ColSummary) BeanUtils.cloneBean(summary);
            ColOperation colExchangeOperation = null;
            if (isBack) {
                switch (operationType) {
                    case 1:// 退回
                    case 100:// 指定回退，流程重走
                        Map<String, Object> businessData = affairData.getBusinessData();
                        Comment comment = new Comment();
                        ParamUtil.getJsonDomainToBean("comment_deal", comment);
                        colExchangeOperation = ColOperation.STEPBACK;
                        break;
                    case 2:// 取回
                        // 取回操作，后续被撤销的事项
                        for (CtpAffair aff : affairs) {
                            aff.setState(StateEnum.col_takeBack.key());
                        }
                        colExchangeOperation = ColOperation.TAKEBACK;
                        break;
                }
            } else if (isCompetition) {
                cloneAffairData.setIsSendMessage(true);
                colExchangeOperation = ColOperation.COMPETE;
            } else if (isDelete) {
                for (CtpAffair aff : affairs) {
                    aff.setState(StateEnum.col_itemRemoved.key());
                    aff.setSubState(SubStateEnum.col_normal.key());
                    aff.setDelete(true);
                }
                colExchangeOperation = ColOperation.DELETE;
            }
            if (colExchangeOperation != null && summary != null) {
                collaborationExchangeManager.handCollaborationAffair(cloneAffairData, colExchangeOperation, cloneSummary);
            }
        }
        return true;
    }

    @Override
    public boolean onWorkitemTakeBack(EventDataContext context) {
        if (AppContext.hasPlugin("ocip")) {
            try {
                CtpAffair affair = null;
                String resource = Global.getConfig("sysCode");
                if (null != context.getBusinessData(WorkFlowEventListener.CTPAFFAIR_CONSTANT)) {
                    affair = (CtpAffair) context.getBusinessData(WorkFlowEventListener.CTPAFFAIR_CONSTANT);
                } else {
                    WorkItem workitem = context.getWorkItem();
                    affair = affairManager.getAffairBySubObjectId(workitem.getId());
                }
                if (affair != null) {
                    AffairData affairData = (AffairData) context.getBusinessData(EventDataContext.CTP_AFFAIR_DATA);
                    if (affairData == null) {
                        affairData = new AffairData();
                    }
                    AffairData cloneAffairData = (AffairData) BeanUtils.cloneBean(affairData);

                    List<CtpAffair> list = affairData.getAffairList();
                    if (Strings.isEmpty(list)) {
                        list = new UniqueList<CtpAffair>();
                    }
                    list.add(affair);
                    list = ocipBizManager.getAffairList4Exchange(list);
                    if (list.isEmpty()) {
                        return false;
                    } else {
                        for (CtpAffair aff : list) {
                            aff.setState(StateEnum.col_pending.key());
                        }
                    }
                    ColSummary summary = null;
                    Object summaryObj = context.getBusinessData(WorkFlowEventListener.COLSUMMARY_CONSTANT);
                    if (summaryObj != null) {
                        summary = (ColSummary) summaryObj;
                    } else {
                        try {
                            summary = colManager.getColSummaryById(affair.getObjectId());
                        } catch (Exception e) {
                            log.error("获取协同主表失败", e);
                        }
                    }
                    cloneAffairData.setAffairList(list);
                    ColSummary cloneSummary = (ColSummary) BeanUtils.cloneBean(summary);
                    if (affair.getApp() == ApplicationCategoryEnum.collaboration.getKey()) {
                        collaborationExchangeManager.handCollaborationAffair(cloneAffairData, ColOperation.TAKEBACK, cloneSummary);
                    }
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return true;
    }

    @Override
    public boolean onProcessCanceled(EventDataContext context) {
        if (AppContext.hasPlugin("ocip")) {
            try {
                Integer _operationType = (Integer) context.getBusinessData().get("operationType");
                String repealComment = "";
                if (null != context.getBusinessData(WorkFlowEventListener.CURRENT_OPERATE_COMMENT_CONTENT)) {
                    repealComment = String.valueOf(context.getBusinessData(WorkFlowEventListener.CURRENT_OPERATE_COMMENT_CONTENT));
                }
                String summaryId = null;
                if (null != context.getBusinessData(WorkFlowEventListener.CURRENT_OPERATE_SUMMARY_ID)) {
                    summaryId = String.valueOf(context.getBusinessData(WorkFlowEventListener.CURRENT_OPERATE_SUMMARY_ID));
                }
                Long _summaryId = null;
                String affairId = String.valueOf(context.getBusinessData(WorkFlowEventListener.CURRENT_OPERATE_AFFAIR_ID));
                Long currentWorkitemId = context.getCurrentWorkitemId();
                CtpAffair currentAffair = null;
                if (Strings.isBlank(affairId) && currentWorkitemId != null) {
                    currentAffair = affairManager.getAffairBySubObjectId(currentWorkitemId);

                } else {
                    currentAffair = affairManager.get(Long.parseLong(affairId));
                }
                if (Strings.isBlank(summaryId)) {
                    _summaryId = currentAffair.getObjectId();
                } else {
                    _summaryId = Long.parseLong(summaryId);
                }
                if (Strings.isNotBlank(repealComment)) {
                    repealComment = repealComment.replaceAll(new String(new char[]{(char) 160}), " ");
                }
                List<Integer> states = new ArrayList<Integer>();
                List<Integer> _states = new ArrayList<Integer>();
                states.add(StateEnum.col_pending.key());
                states.add(StateEnum.col_done.key());
                states.add(StateEnum.col_cancel.key());
                _states.addAll(states);
                List<CtpAffair> affairs = affairManager.getAffairs(_summaryId);
                //对于指定回退由onWorkitemCanceled方法内执行，这里过滤掉
                if (Strings.isNotEmpty(affairs) && _operationType != 100) {
                    List<CtpAffair> _affairs = new ArrayList<CtpAffair>();
                    for (CtpAffair affair : affairs) {
                        if (_states.contains(affair.getState())) {
                            _affairs.add(affair);
                        }
                    }
                    List<CtpAffair> affairList4Exchange = ocipBizManager.getAffairList4Exchange(_affairs);
                    for (CtpAffair affair : affairList4Exchange) {
                        affair.setObjectId(_summaryId);
                        if (affair.getBackFromId() == null) {//设置回退人
                            User currentUser = AppContext.getCurrentUser();
                            OcipOrgMember platformMember = OrgUtil.getPlatformMember(currentUser.getId());
                            if (platformMember != null) {
                                affair.setBackFromId(Long.parseLong(platformMember.getId()));
                            } else {//管理员不在ocip平台直接设置管理员ID（管理员id在数据库初始化时已预设，可跨系统通用）
                                affair.setBackFromId(currentUser.getId());
                            }
                        }
                    }
                    AffairData affairData = new AffairData();
                    affairData.setAffairList(affairList4Exchange);
                    affairData.setIsSendMessage(true);
                    ColSummary summary = null;
                    try {
                        summary = colManager.getColSummaryById(_summaryId);
                    } catch (Exception e) {
                        log.error("获取协同主表失败", e);
                    }
                    ColSummary cloneSummary = (ColSummary) BeanUtils.cloneBean(summary);
                    ColOperation operation = ColOperation.STEPBACK;
                    if (_operationType == WorkFlowEventListener.CANCEL) {
                        operation = ColOperation.REVOKED;
                    }
                    if (currentAffair.getApp() == ApplicationCategoryEnum.collaboration.getKey() && affairList4Exchange.size() > 0) {
                        collaborationExchangeManager.handCollaborationAffair(affairData, operation, cloneSummary, repealComment);
                    }

                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return true;
    }

    @Override
    public boolean onProcessFinished(EventDataContext context) {
        //本想事务完成后再通知一次，但是发送前获取的ColSummary对象还是更新前的，没找到原因，暂时注释掉
        try {

            ColSummary colSummary = null;
            if (context.getAppObject() != null) {
                colSummary = (ColSummary) context.getAppObject();
            }
            if (colSummary == null && context.getProcessId() != null) {
                colSummary = colManager.getColSummaryByProcessId(Long.valueOf(context.getProcessId()));
            }

            List<CtpAffair> affairs = affairManager.getAffairs(ApplicationCategoryEnum.collaboration, colSummary.getId());
            List<CtpAffair> affairsList = new ArrayList<CtpAffair>();
            for (CtpAffair affair : affairs) {
                if (Boolean.TRUE.equals(affair.isDelete())) {
                    continue;
                }
                StateEnum state = StateEnum.valueOf(affair.getState());
                switch (state) {
                    case col_pending:
                    case col_done:
                    case col_competeOver:
                    case col_pending_repeat_auto_deal:
                        affairsList.add(affair);
                        break;
                }
            }
            List<CtpAffair> affairList = ocipBizManager.getAffairList4Exchange(affairsList);
            AffairData affairData = new AffairData();
            affairData.setAffairList(affairList);
            ColSummary cloneSummary = new ColSummary();//传送数据值用
            BeanUtils.copyProperties(cloneSummary, colSummary);
            cloneSummary.setState(CollaborationEnum.flowState.finish.ordinal());
            if (cloneSummary.getFinishDate() == null) {
                cloneSummary.setFinishDate(new Date());
            }
            // 添加标题后缀，区分流程结束和已办
            cloneSummary.setSubject(cloneSummary.getSubject() + "(流程结束通知)");
            // OCIP-2174 外系统事务在外系统人员已办中未显示流程结束标识
            for (CtpAffair affair : affairList) {
                affair.setSummaryState(CollaborationEnum.flowState.finish.ordinal());
            }
            Integer state = colSummary.getState();
            if(Strings.isNotEmpty(affairList) && state == 3){
                collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.DONE, cloneSummary);
            }
            //collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.DONE, cloneSummary);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return true;
    }

    @Override
    public boolean onWorkitemReadyToWait(EventDataContext context) {
        try {
            CtpAffair affair = (CtpAffair) context.getBusinessData("CtpAffair");

            List<CtpAffair> affairs = ocipBizManager.getAffairList4Exchange(affair);
            if (affairs.isEmpty()) {
                return true;
            }
//            for (CtpAffair aff : affairs) {
//                aff.setBackFromId(affair.getMemberId());
//            }
            AffairData data = new AffairData();
            data.setAffairList(affairs);
            ColSummary cloneSummary = new ColSummary();
            ColSummary summary = (ColSummary) context.getAppObject();
            BeanUtils.copyProperties(cloneSummary, summary);
            collaborationExchangeManager.handCollaborationAffair(data, ColOperation.STEPBACK, cloneSummary);
        } catch (Throwable t) {
            log.error(t.getMessage(), t);
        }
        return true;
    }

    @Override
    public boolean onWorkitemDoneToReady(EventDataContext context) {
        try {
            List<WorkItem> workItems = context.getWorkitemLists();
            List<CtpAffair> affairs = new ArrayList<CtpAffair>();
            for (WorkItem item : workItems) {
                CtpAffair affair = this.affairManager.getAffairBySubObjectId(item.getId());
                affairs.add(affair);
            }
            affairs = ocipBizManager.getAffairList4Exchange(affairs);
            if (affairs == null || affairs.isEmpty()) {
                return true;
            }
            AffairData data = new AffairData();
            data.setAffairList(affairs);
            ColSummary cloneSummary = new ColSummary();
            ColSummary summary = (ColSummary) context.getAppObject();
            BeanUtils.copyProperties(cloneSummary, summary);
            collaborationExchangeManager.handCollaborationAffair(data, ColOperation.TODO, cloneSummary);
        } catch (Throwable t) {
            log.error(t.getMessage(), t);
        }
        return true;
    }

    @Override
    public boolean onWorkitemWaitToReady(EventDataContext context) {
        try {
            List<CtpAffair> affairs = (List) context.getBusinessData(ASSIGNED_AFFAIRS);
            affairs = ocipBizManager.getAffairList4Exchange(affairs);
            if (affairs == null || affairs.isEmpty()) {
                return true;
            }
            AffairData data = new AffairData();
            data.setAffairList(affairs);
            ColSummary cloneSummary = new ColSummary();
            ColSummary summary = (ColSummary) context.getAppObject();
            BeanUtils.copyProperties(cloneSummary, summary);
            collaborationExchangeManager.handCollaborationAffair(data, ColOperation.TODO, cloneSummary);

        } catch (Throwable t) {
            log.error(t.getMessage(), t);
        }
        return true;
    }
}
