package com.seeyon.apps.ocip.exchange.collaboration.manager;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.seeyon.apps.agent.bo.MemberAgentBean;
import com.seeyon.apps.collaboration.manager.ColManager;
import com.seeyon.apps.collaboration.po.ColSummary;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.content.affair.AffairData;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.content.comment.Comment;
import com.seeyon.ctp.common.content.comment.CommentManager;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.log.CtpLogFactory;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.po.comment.CtpCommentAll;
import com.seeyon.ctp.common.track.enums.TrackEnum;
import com.seeyon.ctp.common.track.manager.CtpTrackMemberManager;
import com.seeyon.ctp.common.track.po.CtpTrackMember;
import com.seeyon.ctp.common.usermessage.UserMessageUtil;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.UniqueList;
import com.seeyon.ocip.exchange.model.col.ColOperation;
import org.apache.commons.logging.Log;
import org.aspectj.lang.JoinPoint;

public class ColMessageExchangeManager extends OCIPBizManagerImpl {
    private static Log LOG = CtpLogFactory.getLog(ColMessageExchangeManager.class);
    private AffairManager affairManager;
    private ColManager colManager;
    private CommentManager ctpCommentManager;
    private CollaborationExchangeManager collaborationExchangeManager;
    private CtpTrackMemberManager trackManager;

    public void setAffairManager(AffairManager affairManager) {
        this.affairManager = affairManager;
    }

    public void setColManager(ColManager colManager) {
        this.colManager = colManager;
    }

    public void setCollaborationExchangeManager(CollaborationExchangeManager collaborationExchangeManager) {
        this.collaborationExchangeManager = collaborationExchangeManager;
    }

    public void setTrackManager(CtpTrackMemberManager trackManager) {
        this.trackManager = trackManager;
    }

    public void setCtpCommentManager(CommentManager ctpCommentManager) {
        this.ctpCommentManager = ctpCommentManager;
    }

    public void doCommentPushExchangeMessage4SenderNote(JoinPoint joinPoint) {
        if (AppContext.hasPlugin("ocip")) {
            Comment comment = (Comment) joinPoint.getArgs()[0];
            Long summaryId = comment.getModuleId();
            try {
                List<CtpAffair> affairList = affairManager.getValidAffairs(ApplicationCategoryEnum.collaboration, summaryId);
                if (affairList == null || affairList.isEmpty()) {
                    return;
                }
                List<CtpAffair> list = new UniqueList<CtpAffair>(); //用于存储提醒外系统人员意见回复事务
                for (CtpAffair affair : affairList) {
                    Long memberId = affair.getMemberId();
                    Long senderId = affair.getSenderId();
                    if (memberId.intValue() == senderId.intValue()) {
                        continue;
                    }
                    list.add(affair);
                }
                list = getAffairList4Exchange(list);
                if (list == null || list.isEmpty()) {
                    return;
                }
                for (CtpAffair affair : list) {
                    affair.setSubject(comment.getContent());
                    affair.setTempleteId(comment.getId());
                    affair.setAddition(comment.getTitle());
                }
                AffairData affairData = new AffairData();
                affairData.setAffairList(list);
                affairData.setModuleId(comment.getModuleId());
                affairData.setIsSendMessage(false);
                affairData.setSubject(comment.getContent());
                ColSummary colSummary = colManager.getColSummaryById(summaryId);
                collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.COMMENTS, colSummary);
            } catch (Exception e) {
                LOG.error(e);
            }
        }
    }

    public void doCommentPushExchangeMessage4Reply(JoinPoint joinPoint) {
        if (AppContext.hasPlugin("ocip")) {
            User user = AppContext.getCurrentUser();
            Comment comment = (Comment) joinPoint.getArgs()[0];
            Long summaryId = comment.getModuleId();
            try {
                CtpCommentAll comAll = ctpCommentManager.getById(comment.getPid());
                String memberName = getOrgManager().getMemberById(comAll.getCreateId()).getName();
                String commentContent = UserMessageUtil.getComment4Message(Strings.toText(comment.getContent()));
                int commentType = Strings.isTrue(comment.isHidden()) ? 0 : Strings.isBlank(commentContent) ? -1 : 1;
                List<CtpAffair> trackingAffairList = affairManager.getValidTrackAffairs(summaryId);
                List<CtpTrackMember> trackMembers = trackManager.getTrackMembers(summaryId, null);
                trackingAffairList = getTrackAffairExcludePart(trackingAffairList, trackMembers, user.getId());
                List<Long[]> pushMemberIds = comment.getPushMessageToMembersList();
                if (trackingAffairList.isEmpty() && pushMemberIds.isEmpty()) {
                    return;
                }
                String subject = comment.getTitle();
                //用于存储提醒外系统人员意见回复事务
                List<CtpAffair> list = new UniqueList<CtpAffair>();
                for (CtpAffair taffair : trackingAffairList) {
                    if (taffair.isDelete()) {
                        continue;
                    }
                    list.add(taffair);
                }
                for (Long[] push : pushMemberIds) {
                    //判断人员是不是提醒外系统人员,用于修改正文协同交换提醒
                    CtpAffair commentAffair = affairManager.get(push[0]);
                    commentAffair.setMemberId(push[1]);
                    list.add(commentAffair);
                }
                try {
                    list = getAffairList4Exchange(list);
                    //跨系统对意见回复进行提醒
                    if (!list.isEmpty()) {
                        AffairData affairData = new AffairData();
                        affairData.setAffairList(list);
                        affairData.setIsSendMessage(false);
                        affairData.setSubject(subject);
                        affairData.setMemberId(user.getId());//借用存储当前意见回复人员
                        affairData.setForwardMember(user.getName());//借用存储当前意见回复人员姓名
                        affairData.setCaseId(comment.getId());//借用存储意见回复ID
                        affairData.setModuleType(commentType);//借用存储意见回复commentType
                        affairData.setBodyContent(commentContent);//借用存储意见回复内容
                        affairData.setBodyType(memberName);//借用存储意见回复接收人姓名
                        ColSummary colSummary = colManager.getColSummaryById(summaryId);
                        collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.COMMENTREPLY, colSummary);
                    }
                } catch (Exception e) {
                    LOG.error("获取交换事项失败", e);
                }
            } catch (BusinessException e) {
                LOG.error(e);
            }

        }
    }

    /**
     * @param joinPoint
     */
    public void sendExchangeMessage4Zcdb(JoinPoint joinPoint) {
        if (AppContext.hasPlugin("ocip")) {
            CtpAffair zcdbAffair = (CtpAffair) joinPoint.getArgs()[0];
            try {
                List<CtpAffair> list = getAffairList4Exchange(zcdbAffair);
                if (!list.isEmpty()) {
                    ColSummary colSummary = colManager.getColSummaryById(list.get(0).getObjectId());
                    AffairData affairData = new AffairData();
                    affairData.setAffairList(list);
                    affairData.setSubject(colSummary.getSubject());
                    affairData.setIsSendMessage(false);
                    collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.ZCDB, colSummary);
                }
            } catch (Exception e) {

            }
        }
    }

    public void sendExchangeMessage4ModifyBodyOrAtt(JoinPoint joinPoint) {
        if (AppContext.hasPlugin("ocip")) {
//            OCIP-1899  处理事务，加签选人，然后修改正文，点击提交，当前处理节点未处理，加签未成功，但是正文修改成功
            Object[] args = joinPoint.getArgs();
            if (args.length == 0 || !(args[0] instanceof ColSummary)) {
                return;
            }
            Integer type = 0;
            if (args.length > 2 && args[2] instanceof Integer) {
                type = (Integer) args[2];
            }
            ColSummary summary = (ColSummary) args[0];
            CtpAffair af = null;
            if (args.length > 3 && args[3] instanceof CtpAffair) {
                af = (CtpAffair) args[3];
            }
            User user = AppContext.getCurrentUser();
            //获取当前事项的affair集合
            List<StateEnum> states = new ArrayList<StateEnum>();
            states.add(StateEnum.col_pending);
            states.add(StateEnum.col_done);
            states.add(StateEnum.col_sent);
            try {
                List<CtpAffair> affairs;
                if (af != null) {
                    affairs = affairManager.getAffairs(af, states);
                } else {
                    affairs = affairManager.getAffairs(summary.getId(), states);
                }
                //节点人和跟踪节点需要发送消息的人员集合
                List<Long> receiversList = new UniqueList<Long>();
                //用于存储提醒外系统人员超期事务
                List<CtpAffair> list = new UniqueList<CtpAffair>();
                for (CtpAffair affair : affairs) {
                    if (affair.isDelete()) {
                        continue;
                    }
                    //获取代理人
                    Long agentMemberId = MemberAgentBean.getInstance().getAgentMemberId(ApplicationCategoryEnum.collaboration.key(), affair.getMemberId());
                    if (user.getId().equals(agentMemberId)) {
                        if (!receiversList.contains(affair.getMemberId())) {
                            receiversList.add(affair.getMemberId());
                            list.add(affair);
                        }
                        continue;
                    }
                    //给已发节点发送消息
                    Long sendId = affair.getSenderId();
                    if (Integer.valueOf(StateEnum.col_sent.getKey()).equals(affair.getState()) && !receiversList.contains(affair.getSenderId())) {
                        receiversList.add(sendId);
                        list.add(affair);
                    } else if (!receiversList.contains(affair.getMemberId())) {
                        receiversList.add(affair.getMemberId());
                        list.add(affair);
                    }
                }
                try {
                    list = getAffairList4Exchange(list);
                    //跨系统对超期提醒人员进行分发
                    if (!list.isEmpty()) {
                        AffairData affairData = new AffairData();
                        affairData.setAffairList(list);
                        affairData.setIsSendMessage(false);
                        String typeText = "";
                        //0#附件|1#内容|2#内容及附件
                        switch (type) {
                            case 0:
                                typeText = "附件";
                                break;
                            case 1:
                                typeText = "正文";
                                break;
                            case 2:
                                typeText = "正文及附件";
                                break;
                        }
                        affairData.setSubject(String.format("%s（修改%s提醒）", summary.getSubject(), typeText));
                        affairData.setMemberId(user.getId());//借用存储当前修改正文人员
                        affairData.setForwardMember(user.getName());//借用存储当前修改正文人员姓名
                        summary.setColType(type);//借用存储区分是修改正文还是修改附件
                        collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.MODIFYBODY, summary);
                    }
                } catch (Exception e) {
                    LOG.error(e);
                }
            } catch (BusinessException e) {
                LOG.error(e);
            }
        }
    }

    //查找哪些设置了部分跟踪但是没有跟踪当前用户的Affair,从trackAffairs中移除，然后返回跟踪事项列表
    private List<CtpAffair> getTrackAffairExcludePart(List<CtpAffair> trackAffairs, List<CtpTrackMember> trackMembers, Long currentMemberId) {
        for (Iterator<CtpAffair> it = trackAffairs.iterator(); it.hasNext(); ) {
            CtpAffair affair = it.next();
            boolean partTrack = Integer.valueOf(TrackEnum.part.ordinal()).equals(affair.getTrack()); //设置的是部分跟踪
            boolean isTrackCurrentMemebr = false; //设置部分跟踪的时候是否跟踪了当前的用户
            for (CtpTrackMember colTrackMember : trackMembers) {
                if (affair.getId().equals(colTrackMember.getAffairId())) {
                    partTrack = true;
                    if(currentMemberId.equals(colTrackMember.getTrackMemberId())){
                        isTrackCurrentMemebr = true;
                    }
                }
            }
            //设置了部分跟踪但是没有跟踪当前用户的
            if (partTrack && !isTrackCurrentMemebr) {
                it.remove();
            }
        }
        return trackAffairs;
    }
}
