package com.seeyon.apps.ocip.exchange.collaboration.util;

import java.io.UnsupportedEncodingException;
import java.util.Enumeration;
import java.util.LinkedHashMap;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;

import com.seeyon.apps.ocip.util.OrgUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.affair.util.AffairUtil;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.content.affair.constants.SubStateEnum;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.util.Base64;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ocip.common.org.OcipOrgMember;
import com.seeyon.ocip.common.organization.IOrganizationManager;
import com.seeyon.ocip.configuration.OcipConfiguration;
import com.seeyon.ocip.exchange.exceptions.TransportException;
import com.seeyon.ocip.online.OnlineChecker;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.web.servlet.ModelAndView;

public class CollaborationExchageUtil {
    private static final Log log = LogFactory.getLog(CollaborationExchageUtil.class);

    /**
     * 公文和协同穿透都可以走这里
     *
     * @param request
     * @return ModelAndView
     * @throws Exception
     */
    public static ModelAndView getOutCollaborationLink(HttpServletRequest request) throws Exception {
        Map<String, String> paramters = new LinkedHashMap<String, String>();
        Enumeration<String> parameterNames = request.getParameterNames();
        while (parameterNames.hasMoreElements()) {
            String parameterName = parameterNames.nextElement();
            if ("method".equalsIgnoreCase(parameterName)) {
                continue;
            }
            String parameterValue = request.getParameter(parameterName);
            paramters.put(parameterName, parameterValue);
        }
        return getCollaborationModel(paramters);
    }

    public static ModelAndView getCollaborationModel(Map<String, String> paramters) {
        String affairId = paramters.get("affairId");
        if (Strings.isBlank(affairId)) {
            return null;
        }
        paramters.put("affairId", encodeData(affairId));
        IOrganizationManager organizationManager = (IOrganizationManager) AppContext.getBean("organizationManager");
        String link = "";
        String requestAddress = "";
        ModelAndView modelAndView = null;
        try {
            AffairManager affairManager = (AffairManager) AppContext.getBean("affairManager");
            CtpAffair affair = affairManager.get(Long.valueOf(affairId));
            String sendSysCode = "";
            Long memberId = null;
            Map<String, Object> affairExtMap = AffairUtil.getExtProperty(affair);
            if (affairExtMap != null) {
                sendSysCode = (String) affairExtMap.get("ocipSystemCode");
                // 多G6登录使用的人员ID
                memberId = (Long) affairExtMap.get("memberId_of_ocip_client");
            }
            // 本地系统的affair直接跳出
            if (Strings.isBlank(sendSysCode)) {
                return null;
            }
            boolean online = OnlineChecker.isOnline();
            if (!online) {
                // 本地系统没有连接上OCIP
                return rendError(OnlineChecker.OFFLINE_ERROR);
            }
            try {
                requestAddress = OcipConfiguration.getInstance().getExchangeSpi().getTransportService().requestAddress(sendSysCode);
            } catch (Throwable e) {
                if (e instanceof TransportException) {
                    return rendError(((TransportException) e).getErrorMsg());
                }
                return rendError("获取外部系统地址错误，请联系管理员！msg=" + e.getMessage());
            }
            if (Strings.isBlank(requestAddress)) {
                return rendError("获取外部系统地址错误，请联系管理员！");
            }

            Integer sub_state = affair.getSubState();
            if (sub_state == null || sub_state.intValue() == SubStateEnum.col_pending_unRead.key()) {
                affair.setSubState(SubStateEnum.col_pending_read.key());
                affairManager.updateAffair(affair);
            }
            User currentUser = AppContext.getCurrentUser();
            if (memberId == null) {
                if (OrgUtil.isPlatformEntity(currentUser)) {
                    memberId = currentUser.getId();
                } else {
                    String sysCode = OcipConfiguration.getInstance().getSystemIdentification().getId();
                    String currentUserId = String.valueOf(AppContext.getCurrentUser().getId());
                    OcipOrgMember ocipMember = organizationManager.getMember(currentUserId, sysCode);
                    if (ocipMember == null) {
                        memberId = affair.getMemberId();
                    } else {
                        memberId = Long.parseLong(ocipMember.getId());
                    }
                }
            }
            String uId = encodeData(String.valueOf(memberId));
            link = "http://" + requestAddress + "/seeyon/colView.do?ticket=" + uId;
            for (Map.Entry<String, String> entry : paramters.entrySet()) {
                link += "&" + entry.getKey() + "=" + entry.getValue();
            }
            // 代理处理
            Long proxyMemberId = affair.getProxyMemberId();
            if (currentUser.getId().equals(proxyMemberId)) {
                link += "&isAgent=true";
            }
            if (Strings.isNotBlank(link)) {
                modelAndView = new ModelAndView("common/redirect", "redirectURL", link);
            }
        } catch (Exception e) {
            modelAndView = null;
            log.error(e);
        }
        return modelAndView;
    }

    private static ModelAndView rendError(String error) {
        return new ModelAndView().addObject("error", Strings.escapeJavascript(error));
    }

    /**
     * 对给定的字符串进行base64加密操作
     */
    public static String encodeData(String inputData) {
        try {
            if (null == inputData) {
                return null;
            }
            return new String(Base64.encodeBase64(inputData.getBytes("UTF-8"), false));
        } catch (UnsupportedEncodingException e) {
            log.error(inputData, e);
        }
        return null;
    }
}
